%Script to analyse the experimental results produced from Batch_Process_multiple_files.m
%It is possible to process multiple .mat files (each containing multiple experiments) at once.
%Use the settings section to select which data to load
%Use the 'Select specific data' section to only look at specific parts of the loaded data

%% Settings 
clc
clear all
close all

%Enter variables to be subtracted from all experiments
vars = {'results.turbine.Pm';'results.turbine.RPM';'results.p1.drop.calc.amp';'results.Freq.mean';'settings.data.GeneratorResistance';'results.p2.drop.mean';'results.E2.drop.calc';'settings.physics.T0';'settings.physics.P0';'results.E2.S3S4.calc';'results.turbine.eff';'results.p1.S1.mean';'results.p1.S2.mean';'results.p1.S3.mean';'results.p1.S4.mean';'results.phi.S1.mean';'results.phi.S2.mean';'results.phi.S3.mean';'results.phi.S4.mean';'results.generator.relative_power_difference';'results.velocity.ul.complex';'results.velocity.ur.complex';'results.p1.pl';'results.p1.pr';'results.velocity.phi_pQ_l';'results.velocity.phi_pQ_r';'results.E2.drop.mean';'results.E2.drop.calc';'results.E2.drop.tubecorr';'results.E2.S1S2.calc';'results.E2.S3S4.calc';'results.p1.drop.calc.complex'};

remove_invalid_points = 1; %Remove invalid data points, either becuase there is too much electric power deviation or the RPM is out of range

D_selection = 'measured'; %Choose: 'ordered' or 'measured' for the diameter of the rotors

%Add functions to path (must be in the same folder)
mydir = pwd; Functions_path = [mydir '\Functions\'];
addpath(Functions_path);

%Find folder where .mat files are stored (according to the format from the data publication). You can set your own result_folder = 'C:/.....' if you want  
strindex = strfind(mydir,'\'); 
result_folder = [mydir(1:strindex(end)) 'Batch_processed_experimental_data'];

%% Acquire data from results structure of all experiments for all turbine set-ups (i.e. different .mat files)

% Select all .mat files from given folder
mat_files = dir(result_folder); mat_files(1:2) = []; %Get all mat files in the folder. Note that the folder should not contain anything else.
nfiles = length(mat_files);

for k = 1:nfiles %Loop over all files
    current_file = mat_files(k).name;
    current_name = current_file(1:strfind(current_file,'.')-1);
    disp(['---------- Loading results from: ' current_file ' -----------'])
    load([result_folder '\' current_file]) %Load current results
    
    if remove_invalid_points == 1 %Take care to only gather the valid datapoints according to settings.data.setpoint_invalid
        npoints_cumend = cumsum(Batch_metadata.npoints - Batch_metadata.npoints_invalid); %Cumulative index end point for each new experiment
        npoints_cumstart = npoints_cumend-Batch_metadata.npoints+Batch_metadata.npoints_invalid+1; %Cumulative index start point for each new experiment (+1 since MATLAB's first index is 1)

        current_results = zeros(length(vars),sum(Batch_metadata.npoints- Batch_metadata.npoints_invalid)); %Allocate memory for all requested vars for all experiments
        for i=1:Batch_metadata.nfiles %Loop over experiments 
            for j=1:length(vars) %Loop over number of variables to gather
                if length(eval(['Batch_results.experiment' num2str(i) '.' vars{j} ';'])) == 1; %Requested variable has only a single value, such as generator resistance
                     eval(['current_results(j,npoints_cumstart(i):npoints_cumend(i)) = Batch_results.experiment' num2str(i) '.' vars{j} ';']); %Acquire data from structure, use the single value for all points from cumstart to cumend              
                else %Requested variable has a value for each setpoints, such as RPM, Acoustc power etc.
                    eval(['current_results(j,npoints_cumstart(i):npoints_cumend(i)) = Batch_results.experiment' num2str(i) '.' vars{j} '(Batch_results.experiment' num2str(i) '.settings.data.setpoint_invalid==0);']); %Acquire data from structure, only for valid points
                end
            end

        end

    else %Take all data points, even though some might be flagged as invalid in settings.data.setpoint_invalid
        warning('There might be unvalid points. Toggle remove_invalid_points == 1 to remove these points')
        npoints_cumend = cumsum(Batch_metadata.npoints); %Cumulative index end point for each new experiment
        npoints_cumstart = npoints_cumend-Batch_metadata.npoints+1; %Cumulative index start point for each new experiment (+1 since MATLAB's first index is 1)

        current_results = zeros(length(vars),sum(Batch_metadata.npoints)); %Allocate memory for all requested vars for all experiments
        for i=1:Batch_metadata.nfiles %Loop over experiments 
            for j=1:length(vars) %Loop over number of variables to gather
                eval(['current_results(j,npoints_cumstart(i):npoints_cumend(i)) = Batch_results.experiment' num2str(i) '.' vars{j} ';']); %Acquire data from structure
            end

        end
    end
    
    %Store results of current file in results structure
    eval(['results.' current_name '.results = current_results;'])
    eval(['results.' current_name '.metadata = Batch_metadata;'])
    eval(['results.' current_name '.npoints_cumend = npoints_cumend;'])
    eval(['results.' current_name '.npoints_cumstart = npoints_cumstart;'])
    file_names{k} = current_file;
    exp_names{k} = current_name;

end

%% Select specific data from all results
for k=1:nfiles    

eval(['current_results = results.' exp_names{k} '.results;']) 
eval(['npoints_cumend = results.' exp_names{k} '.npoints_cumend;']) 
eval(['npoints_cumstart = results.' exp_names{k} '.npoints_cumstart;']) 
    
points_select_total = cell(length(npoints_cumstart),1);
for i=1:length(npoints_cumstart)
   points_select_total{i} = npoints_cumstart(i):npoints_cumend(i); 
end

%Select data (RPM=variable 2, Frequency=variable 4, Resistance=variable 5)

%For example, directly below means only take data where RPM>1000
select_variable = 2;
requested_value = 1000;
results_select = current_results(:,current_results(select_variable,:)>requested_value); %Only use specific data
indices = current_results(select_variable,:)>requested_value;

% Selection on second variable (use round for frequency and ==)
% select_variable = 4;
% requested_value = 70;
% % % results_select = results_select(:,results_select(select_variable,:)==requested_value); %Only use specific data
% % % indices = current_results(select_variable,:)==requested_value & indices;
% % results_select = results_select(:,results_select(select_variable,:)>requested_value); %Only use specific data
% % indices = current_results(select_variable,:)>requested_value & indices;
% results_select = results_select(:,round(results_select(select_variable,:))==requested_value); %Only use specific data
% indices = round(current_results(select_variable,:))==requested_value & indices;

%Selection on third variable
% select_variable = 5;
% requested_value = 4.7;
% results_select = results_select(:,results_select(select_variable,:)==requested_value); %Only use specific data
% indices = current_results(select_variable,:)==requested_value & indices;
% % results_select = results_select(:,results_select(select_variable,:)<requested_value); %Only use specific data
% % indices = current_results(select_variable,:)<requested_value & indices;

%Find which points have been selected and separate by experiment 
for i=1:length(indices)
    for j=1:length(npoints_cumstart)
        remove_current = points_select_total{j} == i & indices(i) == 0; %If true, found a point that needs to be removed from points_selected cell array since it's not selected 
        points_select_total{j} = points_select_total{j}(~remove_current);
        clear remove_current
    end
end

%Now make new cell array which corresponds to the results_selected
points_select = cell(length(points_select_total),1);

points_select{1} = 1:length(points_select_total{1});
if ~isempty(points_select{1}) %First experiments has selected results
    last_index = points_select{1}(end);
else %First experiments does not have selected results
    last_index = 0;
end
for i=2:length(points_select_total) %Loop over experiments
    points_select{i} =  last_index+1:last_index+length(points_select_total{i});
    if ~isempty(points_select{i}) %Current experiment has selected results
        last_index = points_select{i}(end);
    end
end

eval(['results.' exp_names{k} '.results_select = results_select;'])   
eval(['results.' exp_names{k} '.points_select = points_select;'])   

end

%% Calculate (dimensionless) numbers (scaling based on RPM) 

for k=1:nfiles %Look over all files
    eval(['results_select = results.' exp_names{k} '.results_select;']) %Get current selected results    
 
    dim.rho = rho_0_T(results_select(strcmp(vars,'settings.physics.T0'),:),results_select(strcmp(vars,'settings.physics.P0'),:)); %Account for variable T and P for rho
    dim.mu = mu_T(results_select(strcmp(vars,'settings.physics.T0'),:)); %Account for variable T for mu
    dim.c0 = c_0_T(results_select(strcmp(vars,'settings.physics.T0'),:));

    dim.D = 58/1000; %Diameter of the rotor
    dim.RPM = results_select(2,:);

    %Velocity definition
    dim.Rhub = 19.6/1000; dim.Rtube = 30/1000;
    dim.A_reduction_factor = 1-(dim.Rhub^2)/(dim.Rtube^2);
    dim.va_amp = (abs(results_select(21,:))+abs(results_select(22,:)))/2/dim.A_reduction_factor;
    dim.va_rms = dim.va_amp/sqrt(2);
    dim.va_l_amp = abs(results_select(21,:))/dim.A_reduction_factor; %Axial velocity between hub and tip
    dim.va_l_rms = dim.va_l_amp/sqrt(2);
    dim.Ur = dim.RPM/60*pi*dim.D; %Rotational velocity at tip of the turbine
    
    %Performance indicators
    dim.dP_amp = results_select(3,:);
    dim.dP_rms = dim.dP_amp/sqrt(2);
    
    dim.Pcoef = results_select(1,:)./(dim.rho.*(dim.RPM/60*2*pi).^3*dim.D^5);
    dim.Psi = dim.dP_rms./(dim.rho.*dim.D^2.*(dim.RPM/60*2*pi).^2);
    dim.Re = dim.rho.*(dim.RPM/60*pi).*dim.D^2./dim.mu;
    
    dim.eff = results_select(11,:);

    dim.E2drop = results_select(7,:);
    dim.E2dropcoef = dim.E2drop./(dim.rho.*(dim.RPM/60*pi).^3*dim.D^5);

    dim.disp_amp = dim.va_amp./(2*pi*results_select(4,:)); 
    dim.disp_amp_l = abs(results_select(21,:))./(2*pi*results_select(4,:)); 
    dim.disp_amp_r = abs(results_select(22,:))./(2*pi*results_select(4,:));

    dim.impedance_l = results_select(23,:)./results_select(21,:);
    dim.impedance_r = results_select(24,:)./results_select(22,:);

    %Scaling
    dim.flowcoef = dim.va_rms./dim.Ur;
    dim.E2dropscaling = dim.E2dropcoef./(dim.flowcoef.^3 + dim.flowcoef.^2);
  
    eval(['results.' exp_names{k} '.dim = dim;']) 
end

%% Plot dE2 vs. eff (example of how to plot things)

for k=[1] %You can loop over multiple '.mat' files by settings all desired k indices

eval(['dim = results.' exp_names{k} '.dim;'])  
eval(['eff = results.' exp_names{k} '.dim.eff;'])  
eval(['E2drop = results.' exp_names{k} '.dim.E2drop;'])  
eval(['points_select = results.' exp_names{k} '.points_select;'])
eval(['results_select = results.' exp_names{k} '.results_select;'])

points = [3,4,5,6,13];
j=2; %Select j such that number of frequencies per resistor match

markers = {'.','+','o','s','d','v','d','^','x','>','<','pentagram','hexagram'};

figure('pos',[600 20 800 600])
hold on

lgdstring = {};
for i=1:points(j)
    if ~isempty(points_select{i}) %Current experiment has selected results
        plot(E2drop(points_select{i}),eff(points_select{i}),'Color','b','Marker',markers{mod(i,length(markers))+1},'Linewidth',2,'Markersize',8)
        lgdstring{length(lgdstring)+1} = [num2str(results_select(5,points_select{i}(1))) 'Ohm, ' num2str(results_select(4,points_select{i}(1))) 'Hz'];
    end  
end

for i=points(j)+1:2*points(j)
    if ~isempty(points_select{i}) %Current experiment has selected results
        plot(E2drop(points_select{i}),eff(points_select{i}),'Color','r','Marker',markers{mod(i-points(j),length(markers))+1},'Linewidth',2,'Markersize',8)
        lgdstring{length(lgdstring)+1} = [num2str(results_select(5,points_select{i}(1))) 'Ohm, ' num2str(results_select(4,points_select{i}(1))) 'Hz'];
    end
end

for i=2*points(j)+1:3*points(j)
    if ~isempty(points_select{i}) %Current experiment has selected results
        plot(E2drop(points_select{i}),eff(points_select{i}),'Color','k','Marker',markers{mod(i-2*points(j),length(markers))+1},'Linewidth',2,'Markersize',8)
        lgdstring{length(lgdstring)+1} = [num2str(results_select(5,points_select{i}(1))) 'Ohm, ' num2str(results_select(4,points_select{i}(1))) 'Hz'];
    end
end

yrange = 0.1;
yrange_from_data = max(eff)-min(eff);
yrange_delta = (yrange-yrange_from_data)/2;

axis([min(E2drop)-0.1 max(E2drop)+0.1 min(eff)-yrange_delta max(eff)+yrange_delta])
xlim([0 30])
ylim([0.19 0.26])
set(gca,'Fontsize',16)

xlabel('$\Delta E_2$ [W]','Fontsize',20,'Interpreter','Latex')
ylabel('$\eta_t$','Fontsize',24,'Interpreter','Latex')
title(exp_names{k},'Fontsize',16)

%Uncomment to display legend
% lgd = legend(lgdstring,'Location','SouthEast');
% set(lgd,'FontSize',16); 

end








