%Script to batch process multiple individual experiments using ProcessTurbine_single_TDMS_files.m
%This script will run and save all the important results in a .mat file
%Run the script and select the TDMS data (need to be more than 1 TDMS file)

%% Initialize 
clc
clear all
close all

%Set filename for saving all results (output in same folder as the results)
save_filename = 'mt01test.mat';

%Set name of main processing script
run_script = 'ProcessTurbine_single_TDMS_files';

%Set the variables to be saved after processing each file
settings.data.Batch.variables = {'settings', 'data','results'};

%Select the files to be processed using dialog box (all results should be
%in the same folder). You have to select more than 1 file!
mydir = pwd; settings.data.default_path = mydir;
[settings.data.Batch.filenames, settings.data.Batch.pathname] = uigetfile('*.tdms','Select all batch result files (must be in same folder)',settings.data.default_path,'MultiSelect','On');
settings.data.Batch.nfiles = length(settings.data.Batch.filenames); %Number of files to process

%% Batch processing and saving
disp(['----------------- Start processing ' num2str(settings.data.Batch.nfiles) ' files in batch processing mode -----------------'  ])

npoints = zeros(1,settings.data.Batch.nfiles); %Array to store number of measurement points for each experiment 
npoints_invalid = zeros(1,settings.data.Batch.nfiles); %Array to store number of measurement points found to be invalid for each experiment 
for file_n = 1:settings.data.Batch.nfiles %Loop over the number of files for processing
    current_filename = settings.data.Batch.filenames{file_n};
    
    %Proces the current file
    eval(run_script)
    
    npoints(file_n) = settings.data.Nsetpoint; %Number of measurement points to store in metadata
    npoints_invalid(file_n) = sum(settings.data.setpoint_invalid); %Number of measurement points to store in metadata
    
    %Rename the result variables by affixing file_n to the variable for future saving
    for vars = 1:length(settings.data.Batch.variables)
        eval(['Batch_results.experiment' num2str(file_n) '.'  settings.data.Batch.variables{vars} '=' settings.data.Batch.variables{vars} ';']);
    end 
    disp(['----------------- Finished processing batch file ' num2str(file_n) ' out of ' num2str(settings.data.Batch.nfiles) ' -----------------'  ])
    
    %Clear workspace for next processing run (except for what is needed)
    clearvars -except file_n Batch_results save_filename run_script settings npoints npoints_invalid
end

%Save all data in a single .mat file
Batch_metadata = settings.data.Batch; %Save the Batch file metadata separately (should also be in each individual settings structure)
Batch_metadata.npoints = npoints;
Batch_metadata.npoints_invalid = npoints_invalid;
save([settings.data.Batch.pathname save_filename],'Batch_metadata','Batch_results')
disp(['All results saved in ' settings.data.Batch.pathname save_filename])
