function [dE2Corr,dE2_left,dE2_right]= AcousticPowerCorrection(results,settings)
if settings.data.batch_processing == 0 %Don't show in batch processing mode
    disp('Correcting acoustic power for additional tube lengths...');
end
Nmeas = length(results.Freq.mean);
dE2Corr = NaN*zeros(Nmeas,1);
dE2_left = NaN*zeros(Nmeas,1);
dE2_right = NaN*zeros(Nmeas,1);
for index = 1:Nmeas
    %Contrary to definition in settings.data.PressureLocations, for the calculations here the x is
    %positive away from the speaker, such that the calculations are done in
    %the same direction as the acoustic wave propagates.
    
    % Left of turbine (P1 and P2)
    x1 = settings.data.PressureLocations(1);
    x2 = settings.data.PressureLocations(2);
    dx = x2-x1;
    xMid = (x1+x2)/2;
    L = (settings.data.xturbine_start-xMid)*-1; %*-1 to flip x-direction to positive away from the speaker    
    p_A = MakeComplex(results.p1.S2.mean(index),results.phi.S1S2.mean(index));
    p_B = MakeComplex(results.p1.S1.mean(index),0);
    [p1x, x] = CalcPressureField(p_A, p_B, results.Freq.mean(index), dx, settings);
    p_in = p1x(dsearchn(x',dx/2)); % pressure in between two sensors
    U_in = CalcFlowRate(p_A,p_B,dx,results.Freq.mean(index),settings,settings.geom.R0,'fusco');
    [~, ~, ~, dE2_left(index)] = CalcAcousticField(p_in,U_in,results.Freq.mean(index),L,settings);
    
    % Right of turbine (P3 and P4)
    x1 = settings.data.PressureLocations(3);
    x2 = settings.data.PressureLocations(4);
    dx = x2-x1;
    xMid = (x1+x2)/2;
    L = (settings.data.xturbine_start+settings.geom.Lturbine-xMid)*-1; %*-1 to flip x-direction to positive away from the speaker
    p_A = MakeComplex(results.p1.S4.mean(index),results.phi.S3S4.mean(index));
    p_B = MakeComplex(results.p1.S3.mean(index),0);
    [p1x, x] = CalcPressureField(p_A, p_B, results.Freq.mean(index), dx, settings);
    p_in = p1x(dsearchn(x',dx/2)); % pressure in between two sensors
    U_in = CalcFlowRate(p_A,p_B,dx,results.Freq.mean(index),settings,settings.geom.R0,'fusco');
    [~, ~, ~, dE2_right(index)] = CalcAcousticField(p_in,U_in,results.Freq.mean(index),L,settings);

    dE2Corr(index) = dE2_left(index)+dE2_right(index);
    
end

end