function [results, settings] = CalcTurbine_performance(results,settings)

% Calculates shaft power and efficiency of turbine.
% Generator calibration data used to determine shaft power from measured
% electric power.

%% Calculate turbine efficiency according to the applied resistor load and corresponding generator efficiency
dE2 = results.E2.drop.calc;
RPM = results.generator.frequency.mean*60/6; %Divide by 6 is #of pole pairs of the generator
PE = 3*(results.generator.RMSamp.mean/1000).^2/settings.data.GeneratorResistance;

        switch settings.geom.used_generator
            case 'gen2'
                switch settings.data.GeneratorResistance
                    case 4.7
                        results.turbine.eff = PE./dE2./(-2.229e-16*RPM.^4 + 4.654e-12*RPM.^3 -3.725e-08*RPM.^2 + 0.0001472*RPM + 0.4997);
                        results.turbine.Pm = PE./(-2.229e-16*RPM.^4 + 4.654e-12*RPM.^3 -3.725e-08*RPM.^2 + 0.0001472*RPM + 0.4997);
                        RPM_calibration_range = [1212 6981]; %Specify range in which generator data is valid
                    case 10
                        results.turbine.eff = PE./dE2./(-1.71e-16*RPM.^4 + 4.049e-12*RPM.^3 -3.714e-08*RPM.^2 + 0.0001698*RPM + 0.3948);    
                        results.turbine.Pm = PE./(-1.71e-16*RPM.^4 + 4.049e-12*RPM.^3 -3.714e-08*RPM.^2 + 0.0001698*RPM + 0.3948);
                        RPM_calibration_range = [1310 8208]; %Specify range in which generator data is valid
                    case 20
                        results.turbine.eff = PE./dE2./(-8.686e-17*RPM.^4 + 2.458e-12*RPM.^3 -2.735e-08*RPM.^2 + 0.0001542*RPM + 0.2598); 
                        results.turbine.Pm = PE./(-8.686e-17*RPM.^4 + 2.458e-12*RPM.^3 -2.735e-08*RPM.^2 + 0.0001542*RPM + 0.2598);
                        RPM_calibration_range = [1366 8544]; %Specify range in which generator data is valid
                    otherwise
                        warning('Unknown resistance to generator 2 specified. NO generator efficiency available at this load')
                end
            case 'gen3' 
                switch settings.data.GeneratorResistance
                    %calibration data from Dec2018 (Pico 80Hz)
                    case 4.7
                        results.turbine.eff = PE./dE2./(-2.423e-16*RPM.^4 + 5.106e-12*RPM.^3 -4.174e-08*RPM.^2 + 0.0001669*RPM + 0.4191);
                        results.turbine.Pm = PE./(-2.423e-16*RPM.^4 + 5.106e-12*RPM.^3 -4.174e-08*RPM.^2 + 0.0001669*RPM + 0.4191);
                        RPM_calibration_range = [1351 6834]; %Specify range in which generator data is valid
                    case 10
                        results.turbine.eff = PE./dE2./(-1.87e-16*RPM.^4 + 4.548e-12*RPM.^3 -4.242e-08*RPM.^2 + 0.0001937*RPM + 0.3218);
                        results.turbine.Pm = PE./(-1.87e-16*RPM.^4 + 4.548e-12*RPM.^3 -4.242e-08*RPM.^2 + 0.0001937*RPM + 0.3218);
                        RPM_calibration_range = [1416 8081]; %Specify range in which generator data is valid
                    case 20
                        results.turbine.eff = PE./dE2./(-1.767e-16*RPM.^4 + 4.445e-12*RPM.^3 -4.304e-08*RPM.^2 + 0.0002075*RPM + 0.1766 );
                        results.turbine.Pm = PE./(-1.767e-16*RPM.^4 + 4.445e-12*RPM.^3 -4.304e-08*RPM.^2 + 0.0002075*RPM + 0.1766 );
                        RPM_calibration_range = [1368 8407]; %Specify range in which generator data is valid
                    
                %calibration data from July19
%                     case 4.7
%                         results.turbine.eff = PE./dE2./(-4.38e-16*RPM.^4 + 8.253e-12*RPM.^3 -5.959e-08*RPM.^2 + 0.0002084*RPM + 0.3792);
%                         results.turbine.Pm = PE./(-4.38e-16*RPM.^4 + 8.253e-12*RPM.^3 -5.959e-08*RPM.^2 + 0.0002084*RPM + 0.3792);
%                         RPM_calibration_range = [1225 6750]; %Specify range in which generator data is valid
%                     case 10
%                         results.turbine.eff = PE./dE2./(-6.327e-16*RPM.^4 + 1.129e-11*RPM.^3 -7.905e-08*RPM.^2 + 0.0002785*RPM + 0.2439);
%                         results.turbine.Pm = PE./(-6.327e-16*RPM.^4 + 1.129e-11*RPM.^3 -7.905e-08*RPM.^2 + 0.0002785*RPM + 0.2439);
%                         RPM_calibration_range = [1100 6000]; %Specify range in which generator data is valid
%                     case 20
%                         results.turbine.eff = PE./dE2./(-6.777e-16 *RPM.^4 + 1.179e-11*RPM.^3 -8.073e-08*RPM.^2 + 0.000287*RPM + 0.1051);
%                         results.turbine.Pm = PE./(-6.777e-16 *RPM.^4 + 1.179e-11*RPM.^3 -8.073e-08*RPM.^2 + 0.000287*RPM + 0.1051);
%                         RPM_calibration_range = [1100 5700]; %Specify range in which generator data is valid
                   
                    otherwise
                        warning('Unknown resistance to generator 3 specified. NO generator efficiency available at this load')
                end   
            case 'gen5' 
                switch settings.data.GeneratorResistance
                    case 50
                        results.turbine.eff = PE./dE2./(-2.026e-17*RPM.^4 + 8.303e-13*RPM.^3 -1.293e-08*RPM.^2 + 9.819e-05*RPM + 0.1507);
                        results.turbine.Pm = PE./(-2.026e-17*RPM.^4 + 8.303e-13*RPM.^3 -1.293e-08*RPM.^2 + 9.819e-05*RPM + 0.1507);
                        RPM_calibration_range = [1610 15020]; %Specify range in which generator data is valid
                    case 100
                        results.turbine.eff = PE./dE2./(-1.047e-17*RPM.^4 + 4.555e-13*RPM.^3 -7.662e-09*RPM.^2 + 6.464e-05*RPM + 0.08022);
                        results.turbine.Pm = PE./(-1.047e-17*RPM.^4 + 4.555e-13*RPM.^3 -7.662e-09*RPM.^2 + 6.464e-05*RPM + 0.08022);
                        RPM_calibration_range = [1638 15950]; %Specify range in which generator data is valid                  
                    otherwise
                        warning('Unknown resistance to generator 5 specified. NO generator efficiency available at this load')
                end 
            otherwise
                warning('Unknown generator entered. NO generator efficiency available')
        end

%Store in results structure
results.turbine.RPM = RPM;
results.generator.PE = PE;

%% Check generator power output vs. RPM (if too much deviation from generator calibration data then the measurement is not valid) (store invalid points)
if settings.data.check_gen_output  == 1
    switch settings.geom.used_generator
        case 'gen2'
            switch settings.data.GeneratorResistance
                case 4.7
                    expected_ACpower = 1.344e-07*RPM.^2 + 1.257e-05*RPM -0.01287;
                case 10
                    expected_ACpower = 7.618e-08*RPM.^2 + 8.001e-06*RPM -0.01068;
                case 20
                    expected_ACpower = 4.194e-08*RPM.^2 + 2.892e-06*RPM -0.00411;
                otherwise
                    warning('Unknown resistance to generator specified. NO AC power output available at this load')
            end
        case 'gen3' 
            switch settings.data.GeneratorResistance
                  %calibration data from Dec2018 (Pico 80Hz)
                case 4.7
                    expected_ACpower = 1.252e-07*RPM.^2 + 2.18e-05*RPM -0.02546;
                case 10
                    expected_ACpower = 7.368e-08*RPM.^2 + 1.013e-05*RPM -0.01377;
                case 20
                    expected_ACpower = 4.106e-08*RPM.^2 + 4.482e-06*RPM -0.006102;

                %calibration data from July19
%                 case 4.7
%                     expected_ACpower = 1.239e-07*RPM.^2 + 7.873e-06*RPM -0.006841;
%                 case 10
%                     expected_ACpower = 7.374e-08*RPM.^2 + 1.538e-06*RPM -0.0007712;
%                 case 20
%                     expected_ACpower = 4.055e-08*RPM.^2 + 3.781e-07*RPM -2.902e-05;

                otherwise
                    warning('Unknown resistance to generator specified. NO AC power output available at this load')
            end
        case 'gen5' 
            switch settings.data.GeneratorResistance
               case 50
                    expected_ACpower = 1.937e-08*RPM.^2 + 3.625e-06*RPM -0.007104;
               case 100
                    expected_ACpower = 9.871e-09*RPM.^2 + 2.095e-06*RPM -0.004244;
                otherwise
                    warning('Unknown resistance to generator specified. NO AC power output available at this load')
            end
        otherwise
            warning('Unknown generator entered. NO AC power output data available')
    end
    if settings.data.batch_processing == 0 %Don't show in batch processing mode
%         figure(3)
%         hold all
%         plot(RPM,expected_ACpower,'r.')
%         plot(RPM,PE,'o')
%         xlabel('RPM')
%         ylabel('Electric power output')
    end

    %Relative difference output AC power and expected AC power
    rel_power_difference_percentage = abs(expected_ACpower-PE)./expected_ACpower*100;
    if settings.data.batch_processing == 0 %Don't show in batch processing mode
%         figure(4)
%         hold all
%         plot(RPM,rel_power_difference_percentage,'o')
%         xlabel('RPM')
%         ylabel('Relative electric power difference')
    end
    
    %Check if relative power difference is small enough
    if max(rel_power_difference_percentage) < settings.data.check_gen_output_ACtolerance
       disp(['Generator electric power output accurate enough. Maximum deviation from calibration is: ' num2str(max(rel_power_difference_percentage)) '%']) 
       settings.data.setpoint_invalid = false(1,length(RPM));   %Set none of the points to be invalid
    else %One or several points found with too much deviation. Store the indices of these points, such that they can be excluded in further processing
        settings.data.setpoint_invalid = rel_power_difference_percentage > settings.data.check_gen_output_ACtolerance; %Invalid points stored as ==1, valid == 0
        disp(['WARNING: Generator power output shows too much deviation from the calibration data. Maximum deviation is: ' num2str(max(rel_power_difference_percentage))  '%, which is higher than the maximum value set in settings.data.check_gen_output_ACtolerance. ']) 
        disp(['Therefore, ' num2str(sum(settings.data.setpoint_invalid)) ' point(s) has/have been marked in settings.data.setpoint_invalid to be invalid.'])
    end
    
    %Check if certain powers are too high compared with the average (to filter individual outliers). Tolerance set by: settings.data.check_gen_output_max_delta
    too_large_delta = rel_power_difference_percentage - mean(rel_power_difference_percentage) > settings.data.check_gen_output_max_delta;
    if sum(xor(too_large_delta, settings.data.setpoint_invalid) & too_large_delta) > 0
       disp(['WARNING: Generator power output difference percentage larger than the mean for one or several points, as set by settings.data.check_gen_output_max_delta.'])  
       disp(['Therefore, ' num2str(sum(xor(too_large_delta, settings.data.setpoint_invalid) & too_large_delta)) ' point(s) has/have been marked in settings.data.setpoint_invalid to be invalid.'])
    end
    settings.data.setpoint_invalid = settings.data.setpoint_invalid | too_large_delta;

    %Store in results structure
    results.generator.expected_ACpower = expected_ACpower;
    results.generator.relative_power_difference =  rel_power_difference_percentage;
    
else
    warning('Electric power output of the generator not checked for accuracy. You can toggle this check by settings.data.check_gen_output == 1')
end

%% Check if data is outside of RPM range for which the generator is calibrated (store unvalid points)

if settings.data.check_gen_output  == 1
    Outside_RPM_range = RPM<RPM_calibration_range(1) | RPM>RPM_calibration_range(2); %Setpoints outside of calibration range
    settings.data.setpoint_invalid = settings.data.setpoint_invalid | Outside_RPM_range;
    if sum(Outside_RPM_range) > 0
        disp(['WARNING: There is/are ' num2str(sum(Outside_RPM_range)) ' setpoint(s) outside of the RPM calibration range. These points have been marked in settings.data.setpoint_invalid to be invalid.']) 
    end
    
else
    warning('It is not checked if all RPMs are within the generator calibration range. You can toggle this check by settings.data.check_gen_output == 1')
    settings.data.setpoint_invalid = false(1,length(RPM));   %Set none of the points to be invalid (even though they might be!). This prevents error in further processing if toggle == 0
end


end